#!/home/michelo/progetti/test-chaos/bin/python3

# import pychaos
import sys
import time
import os
import bson
import json
from kafka import KafkaProducer


if len(sys.argv)<4:
    print("You must specify a <broker:port [9092]> <NodeUID> <cmdname> [JSON VALUE]");
    sys.exit(1);

topic=sys.argv[2].replace('/','.')
topic=topic+"_cmd"
print("sending to:"+sys.argv[1]+" topic:"+topic)
cmd={}


producer = KafkaProducer(bootstrap_servers=sys.argv[1],acks='all',batch_size=0)

if producer.bootstrap_connected():
    print("Connected")
else:
    print("cannot connect to:"+sys.argv[1])
    sys.exit(1)

if len(sys.argv)==5:
    cmd=json.loads(sys.argv[4])

if 'bc_alias' not in cmd.keys():
    cmd['bc_alias'] = sys.argv[3]
        
print("command:",cmd)

encoded=bson.dumps(cmd)
producer.send(topic,encoded)
#producer.send(topic,b'ciao')
#producer.flush()

future = producer.send(topic,encoded)
result=future.get(timeout=60)

if result:
    print("sent ok")
else:
    print("## error sending "+cmd )

sys.exit(0)



from pychaos.chaoslib import *
import json


def search(_name,_what,_alive,opts={}):
    """ Search information aboud a !CHAOS node

    Arguments:
        _name : str -- is the substring of what you want search
        _what : str -- one of these: "cu"|"us"|"agent"|"cds"|"webui"|"variable"|"snapshotsof"|"snapshots"|"script"|"zone"|"class" 
        _alive : bool  -- search among alive (true) or all(false)

    Keyword Arguments:
        opts {dict} -- optional parameters (default: {{}})

    Returns:
        dict -- return an array of objects depending on the search
    """    
    opt = {
        "what": _what,
        "alive": _alive
         }
    if isinstance(_name, list): 
        opt["names"]=_name
    else:
        opt['name']=_name
    return mdsBase("search",opt)

def getChannel(_name,channel_id):
    """ Retrive the specified dataset correspoding to a given CU

    Arguments:
        _name (str): devs CU or array of CU
        channel_id (int): channel_id (-1: all,0: output, 1: input, 2:custom,3:system, 4: health, 5 cu alarm, 6 dev alarms,128 status)

    Returns:
        [dict] -- the specified dataset
    """
    if isinstance(_name, list): 
        dev_array=dev_array = ','.join(_name)

    else:
        dev_array=_name


    str_url_cu = "dev=" + dev_array + "&cmd=channel&parm=" + str(channel_id);

    return basicPost("CU",str_url_cu)


def sendCUCmd(_name,cmd,param={}):
    """ Send a command to a specified CU

    Arguments:
        _name : str -- CU unique name
        cmd : str -- command name

    Keyword Arguments:
        param : dict -- Parameters of the command if any (default: {{}})

    Returns:
        dict: -- depending on the command
    """    
    if isinstance(_name, list): 
        dev_array=dev_array = ','.join(_name)

    else:
        dev_array=_name


    str_url_cu = "dev=" + dev_array
    if( isinstance(cmd,dict)):
         str_url_cu += "&cmd="+cmd["cmd"];
         if("mode" in cmd):
            str_url_cu += "&mode="+str(cmd["mode"])
         if("prio" in cmd):
            str_url_cu += "&prio="+str(cmd["prio"]);
         if("sched" in cmd):
            str_url_cu += "&sched="+str(cmd["sched"]);
    else:
        str_url_cu += "&cmd="+cmd;
 

    if(param):
        str_url_cu+= "&parm="+json.dumps(param)
        

    return basicPost("CU",str_url_cu)

def setAttribute(devs,attr,value):
    """ Set an input attribute to the specified value


    Arguments:
        devs : str -- CU name of list of CU names
        attr : str -- name of the input attribute to se
        value : str -- the value of attribute

     
    """    
    parm={}
    parm[attr]=str(value)
    return sendCUCmd(devs,"attr",parm)
       


#!/usr/bin/env python

# import pychaos
import sys
import time
import os
import bson
from kafka import KafkaConsumer


if len(sys.argv)<3:
    print("You must specify a <broker:port [9092]> <NodeUID> [groupid]");
    sys.exit(1);

gid="pyexporter"
if len(sys.argv)>3:
    gid=sys.argv[4]
topic=sys.argv[2].replace('/','.')
topic=topic+"_o"
print("subscribing to:"+topic)

consumer = KafkaConsumer(topic, bootstrap_servers=sys.argv[1], group_id=gid)
for msg in consumer:
            decoded = bson.loads(msg.value)
            if "FRAMEBUFFER" in decoded:
                ''' is an image
                '''
                bin=decoded["FRAMEBUFFER"]
                name=decoded['ndk_uid']
                ext=decoded["FMT"]
                seq=decoded['dpck_seq_id']
                ts=decoded['dpck_ats']
                runid=decoded['cudk_run_id']
                dd=time.strftime("%m-%d-%Y", time.localtime(ts/1000))
                name=name+"/"+dd+"/"   
                if not os.path.exists(name):
                    try:
                        os.makedirs(name)
                    except OSError:
                        print ("Creation of the directory %s failed" % name)
                    else:
                        print ("Successfully created the directory %s " % name)

                hr=time.strftime("%H:%M:%S%Z", time.localtime(ts/1000))

                fname=name+str(runid)+"_"+str(seq)+"_"+hr+ext
                f = open(fname, 'w+b')
                f.write(bin)
                f.close
                print("created:"+fname)
            else:    
                print(decoded)
sys.exit(0);




#!/usr/bin/env python
import pychaos
import sys
import time


if len(sys.argv)<4:
    print("You must specify a <server:port> <PSNAME> <current value>");
    sys.exit(1);
else:
    pychaos.setOptions({"uri":sys.argv[1]})

try:
## get the output channel
    ps_out=pychaos.getChannel(sys.argv[2],0);
    if(ps_out[0]["stby"]):
        ## e' off metti polarity pos
        print(sys.argv[2] + " CLEAR STBY  and polarity pos")
        pychaos.setAttribute(sys.argv[2],"polarity",1)
        pychaos.setAttribute(sys.argv[2],"stby",0)


    pychaos.setAttribute(sys.argv[2],"current",sys.argv[3])
    ## give time to set busy
    print("Wating the command will set busy")

    time.sleep(3)
    while True:
        ps_all=pychaos.getChannel(sys.argv[2],-1);
        if(ps_all[0]["system"]["busy"] == False):
            print("Operation Ended")
            print(ps_all)
            break;
        else:
            print("."),
            sys.stdout.flush()


except pychaos.ChaosException as err:
    print("EXCEPTION:",err)



import requests
import json

options = {
    "uri":"localhost:8081",
    "timeout":5000,
}
def setOptions(opt):
    global options
    options = opt

def timestampMillisec64():
    return int((datetime.datetime.utcnow() - datetime.datetime(1970, 1, 1)).total_seconds() * 1000)


class ChaosException(Exception):
    """ Chaos Error/Exception raised for rest exception.

    Attributes:
        message -- explanation of the error
    """

    def __init__(self, message):
        self.message = message

def basicPost(func,params):
    """ Basic post for chaos rest calls
    Args:
        func: string that defines the target function (CU or MDS (administration))
        params: dict with parameters
    
    Raises:
        ChaosException in case of error from !CHAOS or other exception if communication error

    """
    srv = options["uri"];
    url = "http://" + srv + "/" + func;
    r = requests.post(url, data=params, headers={'Content-Type': 'application/x-www-form-urlencoded'})
    if(r.status_code == requests.codes.ok):
        return json.loads(r.text)
    else:
        raise ChaosException(r.text)
  
def mdsBase(cmd,opt):
    """ base function for MDS administration 

    Arguments:
        cmd {[string]} -- commands
        opt {[dict]} -- parameters

    Returns:
        [dict] -- an object depending from the operation
    """    
    param = "cmd=" + cmd + "&parm=" + json.dumps(opt)
    return basicPost("MDS", param)



from setuptools import setup, find_packages, Extension

# Note to self: To upload a new version to PyPI, run:
# pip install wheel twine
# python3 setup.py sdist bdist_wheel
# twine upload dist/*


setup(
    name='pychaos',
    version='1.0.11',
    description='A Python extension module that wraps not !CHAOS REST I/O',
    long_description=open('README.md').read(),
    long_description_content_type="text/markdown",
    author='Andrea Michelotti',
    author_email='andrea.michelotti@lnf.infn.it',
    url='https://baltig.infn.it/chaos-lnf-control/pychaos',
    packages=find_packages(),
    license='CC0 (copyright waived)',
    keywords="CHAOS,CONTROL, REST",
    install_requires=['requests','kafka-python','bson'],
    scripts=["test/pyPS.py","test/pyLive.py","test/pySubscribeNode.py","test/pySendCommand.py"],
    classifiers=[
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    test_suite='tests',
)



#!/usr/bin/env python

import pychaos
import sys


if len(sys.argv)<2:
    print("You must specify a <server:port>");
    sys.exit(1);
else:
    pychaos.setOptions({"uri":sys.argv[1]})
lst=pychaos.search("","cu",True)
print("list of  Alive CU:")
print(lst)

print("channel of:"+lst[0])
lst=pychaos.getChannel(lst[0],-1);
print(lst[0])


'''
cnt=0;

QUESTA DA ERRORE ULTIMA ITERAZIONE?
dnames = ['output','input','custom','system','health','devalarm','cualarm']

dnames = ['output']
for dataset in dnames :
    data=pyChaos.GetJsonDataset(sys.argv[1],cnt);
    cnt+=1
    print data;
'''
sys.exit(0);

